import sys
import ssl
import email
import tempfile
import requests
from imapclient import IMAPClient


def upload_document(
    filepath,
    file_title,
    papermerge_url,
    api_key,
    logger
):
    url = f"{papermerge_url}/api/document/upload/file1.pdf"
    headers = {
        'Authorization': f"Token {api_key}",
        "Content-Type": "application/binary",
    }
    data = open(filepath, 'rb').read()
    try:
        ret = requests.put(
            url,
            headers=headers,
            data=data
        )
        if ret.status_code == 401:
            logger.error(f"Upload failed: {ret.reason}")

        logger.info(
            f"status: {ret.status_code}; reason: {ret.reason}"
        )
    except requests.exceptions.ConnectionError:
        logger.error(f"Failed to connect to {papermerge_url}")
        sys.exit(1)


def read_email_message(
    message,
    papermerge_url,
    api_key,
    logger
):
    """
    message is an instance of python's module email.message
    """
    imported_count = 0

    for part in message.walk():
        # search for payload
        maintype = part.get_content_maintype()
        subtype = part.get_content_subtype()
        if maintype == 'application' and subtype == 'pdf':

            with tempfile.NamedTemporaryFile() as temp:
                temp.write(part.get_payload(decode=True))
                temp.flush()
                #    filepath=temp.name,
                #    file_title=part.get_filename(),
                # Upload document via api_key
                # something like curl -X POST -d "@file"  <server>/api/document/upload/file1 'Authentication: Token <token>'
                upload_document(
                    filepath=temp.name,
                    file_title=part.get_filename,
                    papermerge_url=papermerge_url,
                    api_key=api_key,
                    logger=logger
                )
                imported_count += 1

    return imported_count


def import_attachment(
    config,
    logger
):

    imported_count = 0

    ssl_context = ssl.create_default_context()
    ssl_context.check_hostname = False
    ssl_context.verify_mode = ssl.CERT_NONE

    with IMAPClient(
        config['imap_server'],
        ssl_context=ssl_context
    ) as server:
        server.login(config['username'], config['password'])
        server.select_folder('INBOX')
        messages = server.search(['UNSEEN'])

        logger.debug(
            f"IMAP UNSEEN messages {len(messages)}"
            f" for {config['username']}"
        )

        for uid, message_data in server.fetch(
            messages, 'RFC822'
        ).items():
            email_message = email.message_from_bytes(
                message_data[b'RFC822']
            )
            imported_count = read_email_message(
                email_message,
                config['papermerge_url'],
                config['api_key'],
                logger
            )

    return imported_count



import yaml
import logging.config
import os
import argparse
import logging
import sys
import traceback
import importlib.util
import importlib.machinery

from mgmail.attachment import import_attachment


with open('mgmail.logging.yml', "r") as f:
    config = yaml.safe_load(f)
    logging.config.dictConfig(config)

logger = logging.getLogger(__name__)


class Application:

    def __init__(self):
        self.cfg = None
        self.load_config()
        self.logger = logging.getLogger(__name__)

    def load_config(self):
        parser = argparse.ArgumentParser(prog='mgmail')
        parser.add_argument(
            "--config", nargs=1
        )
        args = parser.parse_args()
        if not args.config:
            config = "mgmail.config.py"
        else:
            config = args.config

        self.cfg = self.load_config_from_filename(config)

    def load_config_from_filename(self, filename):

        if not os.path.exists(filename):
            raise RuntimeError("%r doesn't exist" % filename)

        ext = os.path.splitext(filename)[1]

        try:
            module_name = '__config__'
            if ext in [".py", ".pyc"]:
                spec = importlib.util.spec_from_file_location(
                    module_name,
                    filename
                )
            else:
                msg = "config file should have a valid Python extension.\n"
                self.logger.warn(msg)
                loader_ = importlib.machinery.SourceFileLoader(
                    module_name,
                    filename
                )
                spec = importlib.util.spec_from_file_location(
                    module_name,
                    filename,
                    loader=loader_
                )
            mod = importlib.util.module_from_spec(spec)
            sys.modules[module_name] = mod
            spec.loader.exec_module(mod)
        except Exception:
            print("Failed to read config file: %s" % filename, file=sys.stderr)
            traceback.print_exc()
            sys.stderr.flush()
            sys.exit(1)

        return vars(mod)

    def run(self):
        read_count = import_attachment(
            config=self.cfg,
            logger=self.logger
        )
        self.logger.info(f"Total messages read {read_count}")


def run():
    Application().run()


if __name__ == '__main__':
    run()
